#powershell strongpw

function StrongPwInternal_SetClipboard {
    ## From Windows PowerShell Cookbook (O'Reilly)
    ## by Lee Holmes (http://www.leeholmes.com/guide)

    param(
        [Parameter(ValueFromPipeline = $true)]
        [object[]] $InputObject
    )
    begin
    {
        Set-StrictMode -Version Latest
        $objectsToProcess = @()
    }
    process
    {
        ## Collect everything sent to the script either through
        ## pipeline input, or direct input.
        $objectsToProcess += $inputObject
    }
    end
    {
        ## Launch a new instance of PowerShell in STA mode.
        ## This lets us interact with the Windows clipboard.
        $objectsToProcess | PowerShell -NoProfile -STA -Command {
            Add-Type -Assembly PresentationCore
            ## Convert the input objects to a string representation
            $clipText = ($input | Out-String -Stream) -join "`r`n"
            ## And finally set the clipboard text
            [Windows.Clipboard]::SetText($clipText)
        }
    }
}

function StrongPwInternal_ToSha256 {
    <#

    .SYNOPSIS

    Computes the SHA256 sum from a given input string.
    The result is a byte array.

    .EXAMPLE

    StrongPwInternal_ToSha256 "abc"

    .EXAMPLE

    Dir | StrongPwInternal_ToSha256

    #>
    param(
          ## String from which to compute sha256 sum
          [Parameter(ValueFromPipeline = $true)]
          [string] $InputString
         )

    $utf8_encoder = [System.Text.Encoding]::UTF8
    $input_as_byte_array = $utf8_encoder.GetBytes($InputString)

    $sha256_computation_obj = New-Object -TypeName System.Security.Cryptography.Sha256CryptoServiceProvider
    $sha256_sum = $sha256_computation_obj.ComputeHash($input_as_byte_array)

    return $sha256_sum
}


function StrongPwInternal_ToBase64 {
    <#

    .SYNOPSIS

    Computes the base64 encoded value from a given byte array.
    The result is a string.

    .EXAMPLE

    StrongPwInternal_ToBase64 (25, 25)

    .EXAMPLE

    StrongPwInternal_ToSha256 "abc" | StrongPwInternal_ToBase64

    #>
    param(
          ## byte array from which to compute base64
          [Parameter(ValueFromPipeline = $true)]
          [byte[]] $ByteArray
         )

    return [System.Convert]::ToBase64String($ByteArray)
}

function Get-StrongPw {
    <#

    .SYNOPSIS

    Computes a secure password for one or more specific site(s), given a
    user-provided key.
    There is a user prompt for the key.

    .EXAMPLE

    StrongPw paypal

    .EXAMPLE

    StrongPw paypal facebook

    #>
    if($args.length -eq 0) { return }

    $key = Read-Host -Prompt "Encryption key:" -AsSecureString
    $strkey = [Runtime.InteropServices.Marshal]::PtrToStringAuto([Runtime.InteropServices.Marshal]::SecureStringToBSTR($key))

    foreach ($i in $args)
    {
        $site = $i
        $pw = [string](StrongPwInternal_ToBase64 -ByteArray (StrongPwInternal_ToSha256 -InputString "${strkey}:${site}")).Replace("+","E").Replace("/","a").Substring(0,20)
        if ($args.length -eq 1)
        {
            StrongPwInternal_SetClipboard $pw
            Write-Host "Password for ${site}: (in clipboard)"
        } else
        {
            Write-Host "Password for ${site}: ${pw}"
        }
    }

    $pw = [string](StrongPwInternal_ToBase64 -ByteArray (StrongPwInternal_ToSha256 -InputString ":${strkey}:")).Replace("+","E").Replace("/","a").Substring(0,20)
    Write-Host "Verification code: ${pw}"
}

Export-ModuleMember Get-StrongPw
